@@ -59,15 +59,16 @@ static char *msg_table[] = {
   "password status display for all users denied - by=%u",
   "password status displayed for all users - by=%u",
   /* 30 */ "user is unknown - user=%s, by=%u",
-  "password change denied - user=%s, uid=%u, by=%u",
-  "password status displayed - user=%s, uid=%u, by=%u",
-  "password change failed, pam error %d - user=%s, uid=%u, by=%u",
-  "password changed - user=%s, uid=%u, by=%u",
-  "cannot determine user name - uid=%u",
-  "defaults changed - gid=%u, home=%s, shell=%s, inactive=%li, expire=%s, by=%u",
-  "updating default config file `%s' faild - by=%u",
-  "UID %u is not unique - by=%u",
-  "invalid name - account=%s, by=%u",
+  /* 31 */ "password change denied - user=%s, uid=%u, by=%u",
+  /* 32 */ "password status displayed - user=%s, uid=%u, by=%u",
+  /* 33 */ "password change failed, pam error %d - user=%s, uid=%u, by=%u",
+  /* 34 */ "password changed - user=%s, uid=%u, by=%u",
+  /* 35 */ "can't get unique uid in range %u - %u",
+  /* 36 */ "cannot determine user name - uid=%u",
+  /* 37 */ "defaults changed - gid=%u, home=%s, shell=%s, inactive=%li, expire=%s, by=%u",
+  /* 38 */ "updating default config file `%s' faild - by=%u",
+  /* 39 */ "UID %u is not unique - by=%u",
+  /* 40 */ "invalid name - account=%s, by=%u",
   "account already exists - account=%s, by=%u",
   "new account added - account=%s, uid=%u, gid=%u, home=%s, shell=%s, by=%u",
   "home directory created - account=%s, uid=%u, home=%s, by=%u",
@@ -76,7 +77,12 @@ static char *msg_table[] = {
   "`%s' is not owned by user, not removed - account=%s, uid=%u, by %u",
   "home directory deleted - account=%s, uid=%u, home=%s, by=%u",
   "failed to delete account - account=%s, uid=%u, by=%u",
-  "account deleted - account=%s, uid=%u, by=%u"};
+  "account deleted - account=%s, uid=%u, by=%u",
+  /* 50 */ "error renaming account in group - account=%s, uid=%u, group=%s, gid=%u, by=%u",
+  /* 51 */ "account name renamed in group - new=%s, old=%s, uid=%u, group=%s, gid=%u, by=%u",
+  /* 52 */ "failed to modify account - account=%s, uid=%u, by=%u",
+  /* 53 */ "account is currently in use - account=%s, uid=%u, by=%u",
+  /* 54 */ "account name changed - new=%s, old=%s, uid=%u, by=%u"};
 
 void (*sec_log_fnc) (const char *program, unsigned int id, ...) = NULL;
 
