@@ -77,6 +77,11 @@ extern void (*sec_log_fnc) (const char *program, unsigned int id, ...);
 #define MSG_NOT_OWNED_BY_USER 46 /* file, account, uid, caller uid */
 #define MSG_HOMEDIR_REMOVED 47 /* account, uid, home, caller uid */
 #define MSG_ERROR_REMOVING_USER 48 /* account, uid, caller uid */
-#define MSG_USER_DELETED 21 /* account, uid, caller uid */
+#define MSG_USER_DELETED 49 /* account, uid, caller uid */
+#define MSG_ERROR_RENAME_USER_IN_GROUP 50 /* account, uid, group, gid, caller uid */
+#define MSG_USER_RENAMED_IN_GROUP 51 /* account, old account, uid, group, gid, caller uid */
+#define MSG_ERROR_MODIFYING_USER 52 /* account, uid, caller uid */
+#define MSG_ACCOUNT_IN_USE 53 /* account, uid, caller uid */
+#define MSG_USER_NAME_CHANGED 54 /* new account, old account, uid, caller uid */
 
 #endif
