@@ -160,31 +160,6 @@ files_getgrgid (gid_t gid)
     return NULL;
 }
 
-/* XXX move to libpwdutils.  */
-static char **
-remove_gr_mem (const char *name, char **gr_mem)
-{
-  char **groups;
-  unsigned int i, j;
-
-  for (i = 0; gr_mem[i]; i++) ;
-  ++i;                          /* for trailing NULL pointer */
-
-  groups = malloc (i * sizeof (char *));
-  for (i = 0, j = 0; gr_mem[i]; i++)
-    {
-      if (strcmp (name, gr_mem[i]) != 0)
-        {
-          groups[j] = strdup (gr_mem[i]);
-          ++j;
-        }
-    }
-
-  groups[j] = NULL;
-
-  return groups;
-}
-
 /* XXX move to libpwdutils.  */
 static char **
 add_gr_mem (const char *name, char **gr_mem)
