@@ -168,11 +168,19 @@ files_getpwnam (const char *name)
 {
   enum nss_status status;
   static int buflen = 256;
-  static char *buffer = NULL;
+  static char *buffer;
   static struct passwd resultbuf;
 
   if (buffer == NULL)
-    buffer = malloc (buflen);
+    {
+      buffer = malloc (buflen);
+
+      if (buffer == NULL)
+	{
+	  fputs ("running out of memory!\n", stderr);
+	  exit (E_FAILURE);
+	}
+    }
 
   while ((status =
           files_getpwnam_r (name, &resultbuf, buffer, buflen,
@@ -180,7 +188,7 @@ files_getpwnam (const char *name)
          && errno == ERANGE)
     {
       errno = 0;
-      buflen += 256;
+      buflen *= 2;
       buffer = realloc (buffer, buflen);
     }
   if (status == NSS_STATUS_SUCCESS)
@@ -204,9 +212,15 @@ loop_over_group_file (int quiet)
   long i;
   char *inputname = alloca (strlen (files_etc_dir) + 8);
   char *outputname = alloca (strlen (files_etc_dir) + 20);
-  int bufferlen = 256;
+  int bufferlen = 512;
   char *buffer = malloc (bufferlen);
 
+  if (buffer == NULL)
+    {
+      fputs ("running out of memory!\n", stderr);
+      exit (E_FAILURE);
+    }
+
   strcpy (inputname, files_etc_dir);
   strcat (inputname, "/group");
   strcpy (outputname, files_etc_dir);
@@ -341,7 +355,7 @@ loop_over_group_file (int quiet)
 	       errno == ERANGE)
 	  {
 	    errno = 0;
-	    bufferlen += 256;
+	    bufferlen *= 2;
 	    buffer = realloc (buffer, bufferlen);
 	  }
 
@@ -449,7 +463,7 @@ static struct group *
 files_getgrent (void)
 {
   enum nss_status status;
-  int buflen = 256;
+  int buflen = 512;
   char *buffer = NULL;
   struct group *resultbuf = malloc (sizeof (struct group));
 
@@ -460,7 +474,7 @@ files_getgrent (void)
          == NSS_STATUS_TRYAGAIN && errno == ERANGE)
     {
       errno = 0;
-      buflen += 256;
+      buflen *= 2;
       buffer = realloc (buffer, buflen);
     }
   if (status == NSS_STATUS_SUCCESS)
