@@ -210,16 +210,30 @@ main (int argc, char *argv[])
 	    is_member = 1;
 	  else
 	    {
+	      struct group *g;
 	      char **gp;
 
-	      for (gp = grp->gr_mem; *gp != NULL; gp++)
+	      /* Normally it is enough to check only, if the user is in
+		 grp->gr_mem. But some people split the groups into multiple
+		 one with the same name, but different members. So we have
+		 to step through all group entries and search for the correct
+		 one.  */
+	      setgrent ();
+	      while (!is_member && (g = getgrent ()) != NULL)
 		{
-		  if (strcmp (pw->pw_name, *gp) == 0)
+		  if (g->gr_gid != grp->gr_gid)
+		    continue;
+
+		  for (gp = g->gr_mem; *gp != NULL; gp++)
 		    {
-		      is_member = 1;
-		      break;
+		      if (strcmp (pw->pw_name, *gp) == 0)
+			{
+			  is_member = 1;
+			  break;
+			}
 		    }
 		}
+	      endgrent ();
 	    }
 
 	  if (!is_member && grp->gr_passwd && grp->gr_passwd[0] != '\0')
