@@ -38,12 +38,16 @@
 #include <selinux/selinux.h>
 #include <selinux/av_permissions.h>
 #endif
+#ifdef HAVE_LIBNSCD_H
+#include <libnscd.h>
+#endif
 
 #ifndef _
 #define _(String) gettext (String)
 #endif
 
 #include "public.h"
+#include "logging.h"
 #include "logindefs.h"
 #include "read-files.h"
 #include "error_codes.h"
@@ -339,7 +343,7 @@ passwd_main (const char *program, int argc, char **argv)
   /* For admin only commands we need a user name */
   if (argc == 0 && admin_only)
     {
-      fprintf (stderr, _("%s: user argument missing\n"), program);
+      fprintf (stderr, _("%s: User argument missing\n"), program);
       print_error (program);
       return E_USAGE;
     }
@@ -357,9 +361,10 @@ passwd_main (const char *program, int argc, char **argv)
       if (uid != 0)
         {
           fprintf (stderr, _("%s: Permission denied.\n"), program);
+	  sec_log (program, MSG_PASSWORD_STATUS_FOR_ALL_DENIED, uid);
           return E_NOPERM;
         }
-
+      sec_log (program, MSG_DISPLAY_PASSWORD_STATUS_FOR_ALL, uid);
       setpwent ();
       while ((pw = getpwent ()))
         display_pw (pw);
@@ -384,6 +389,7 @@ passwd_main (const char *program, int argc, char **argv)
 	}
       if (!pw)
 	{
+          sec_log (program, MSG_NO_ACCOUNT_FOUND, uid);
 	  fprintf (stderr, _("%s: Cannot determine your user name.\n"),
 		   program);
 	  return E_NOPERM;
@@ -412,28 +418,24 @@ passwd_main (const char *program, int argc, char **argv)
       pw_data = do_getpwnam (pw->pw_name, use_service);
       if (pw_data == NULL || pw_data->service == S_NONE)
 	{
+	  sec_log (program, MSG_UNKNOWN_USER, pw->pw_name, uid);
 	  if (use_service)
-	    {
-	      fprintf (stderr,
-		       _("%s: User `%s' is not known to service `%s'\n"),
-		       program, pw->pw_name, use_service);
-	      return E_NOPERM;
-	    }
+	    fprintf (stderr,
+		     _("%s: User `%s' is not known to service `%s'\n"),
+		     program, pw->pw_name, use_service);
 	  else if (admin_only)
-	    {
-	      /* Only print error, if we need the pw_data informations
-		 later. Else ignore it and let PAM do it.  */
-	      fprintf (stderr, _("%s: Unknown user `%s'.\n"), program,
-		       pw->pw_name);
-	      return E_NOPERM;
-	    }
+	    /* Only print error, if we need the pw_data informations
+	       later. Else ignore it and let PAM do it.  */
+	    fprintf (stderr, _("%s: Unknown user `%s'.\n"), program,
+		     pw->pw_name);
+	  return E_NOPERM;
 	}
     }
 
 #ifdef WITH_SELINUX
   if (is_selinux_enabled () > 0)
     {
-      if ((getuid () == 0) &&
+      if ((uid == 0) &&
 	  (selinux_check_access (pw->pw_name, PASSWD__PASSWD) != 0))
 	{
 	  security_context_t user_context;
@@ -468,6 +470,8 @@ passwd_main (const char *program, int argc, char **argv)
     {
       if (admin_only)
 	{
+	  sec_log (program, MSG_PASSWORD_CHANGE_DENIED,
+		   pw_data->pw.pw_name, pw_data->pw.pw_uid, uid);
           fprintf(stderr, _("%s: Permission denied\n"), program);
 	  if (pw_data)
 	    free_user_t (pw_data);
@@ -475,6 +479,8 @@ passwd_main (const char *program, int argc, char **argv)
 	}
       if (pw->pw_uid != uid)
 	{
+	  sec_log (program, MSG_PASSWORD_CHANGE_DENIED,
+		   pw_data->pw.pw_name, pw_data->pw.pw_uid, uid);
 	  fprintf (stderr, _("You cannot change the shadow data for `%s'.\n"),
 		   pw->pw_name);
 	  syslog (LOG_WARNING, "%d cannot change shadow data for `%s'",
@@ -487,6 +493,8 @@ passwd_main (const char *program, int argc, char **argv)
 
   if (S_flag)
     {
+      sec_log (program, MSG_DISPLAY_PASSWORD_STATUS,
+	       pw_data->pw.pw_name, pw_data->pw.pw_uid, uid);
       display_pw (pw);
       if (pw_data)
 	free_user_t (pw_data);
@@ -531,6 +539,8 @@ passwd_main (const char *program, int argc, char **argv)
 	{
           syslog (LOG_ERR, "User %s: %s", caller_name,
                   pam_strerror (pamh, ret));
+	  sec_log (program, MSG_PASSWORD_CHANGE_FAILED,
+		   ret, pw_data->pw.pw_name, pw_data->pw.pw_uid, uid);
           sleep (getlogindefs_num ("FAIL_DELAY", 1));
 	  fprintf (stderr, "%s: %s\n", program, pam_strerror (pamh, ret));
 	  if (pw_data)
@@ -539,6 +549,9 @@ passwd_main (const char *program, int argc, char **argv)
 	}
 
       pam_end (pamh, PAM_SUCCESS);
+      sec_log (program, MSG_PASSWORD_CHANGED,
+	       pw_data->pw.pw_name, pw_data->pw.pw_uid, uid);
+      free_user_t (pw_data);
       return E_SUCCESS;
     }
 
@@ -567,6 +580,7 @@ passwd_main (const char *program, int argc, char **argv)
 	    {
 	      fprintf (stderr, _("Cannot unlock the password for `%s'!\n"),
 		       pw_data->pw.pw_name);
+	      free_user_t (pw_data);
 	      return E_FAILURE;
 	    }
 	}
@@ -580,6 +594,7 @@ passwd_main (const char *program, int argc, char **argv)
 	    {
 	      fprintf (stderr, _("Cannot unlock the password for `%s'!\n"),
 		       pw_data->pw.pw_name);
+	      free_user_t (pw_data);
 	      return E_FAILURE;
 	    }
 	}
@@ -601,6 +616,7 @@ passwd_main (const char *program, int argc, char **argv)
 	    {
 	      fprintf (stderr, _("Password for `%s' is already locked!\n"),
 		       pw_data->pw.pw_name);
+	      free_user_t (pw_data);
 	      return E_FAILURE;
 	    }
 	}
@@ -619,6 +635,7 @@ passwd_main (const char *program, int argc, char **argv)
 	    {
 	      fprintf (stderr, _("Password for `%s' is already locked!\n"),
 		       pw_data->pw.pw_name);
+	      free_user_t (pw_data);
 	      return E_FAILURE;
 	    }
 	}
@@ -649,7 +666,9 @@ passwd_main (const char *program, int argc, char **argv)
     }
   else
     {
+#ifdef HAVE_NSCD_FLUSH_CACHE
       nscd_flush_cache ("passwd");
+#endif
       if (!silent)
 	{
 	  if (pw_data->sp_changed)
@@ -659,6 +678,9 @@ passwd_main (const char *program, int argc, char **argv)
 	}
     }
 
+  sec_log (program, MSG_PASSWORD_CHANGED,
+	   pw_data->pw.pw_name, pw_data->pw.pw_uid, uid);
+
   free_user_t (pw_data);
 
   return E_SUCCESS;
@@ -676,6 +698,8 @@ main (int argc, char **argv)
 
   prog = basename (argv[0]);
 
+  open_sec_log ();
+
   openlog (prog, LOG_PID, LOG_AUTHPRIV);
 
   /* Before going any further, raise the ulimit and ignore
