@@ -32,6 +32,9 @@
 #include <locale.h>
 #include <libintl.h>
 #include <sys/stat.h>
+#ifdef HAVE_LIBNSCD_H
+#include <libnscd.h>
+#endif
 
 #include "public.h"
 #include "logindefs.h"
@@ -217,7 +220,9 @@ main (int argc, char *argv[])
 	}
       free_user_t (pw_data);
     }
+#ifdef HAVE_NSCD_FLUSH_CACHE
   nscd_flush_cache ("passwd");
+#endif
 
   /* Rename original shadow file to shadow.old.  */
   {
