@@ -42,12 +42,16 @@
 #include <signal.h>
 #include <string.h>
 
+#ifdef USE_GNUTLS
+#include <gnutls/gnutls.h>
+#else
 #include <openssl/rsa.h>       /* SSLeay stuff */
 #include <openssl/crypto.h>
 #include <openssl/x509.h>
 #include <openssl/pem.h>
 #include <openssl/ssl.h>
 #include <openssl/err.h>
+#endif
 
 #include <security/pam_appl.h>
 #include <security/pam_misc.h>
@@ -72,7 +76,11 @@
 extern int setresuid(uid_t ruid, uid_t euid, uid_t suid);
 
 /* XXX This variable should not be global.  */
+#ifdef USE_GNUTLS
+/* XXX */
+#else
 static SSL_CTX *ctx;
+#endif
 /* Socket for incoming connections.  */
 static struct pollfd pollfd_conn[10];
 static int pollfd_cnt = 0;
@@ -294,6 +302,75 @@ close_sockets (void)
       close (pollfd_conn[i].fd);
 }
 
+#ifdef USE_GNUTLS
+static int
+send_string (gnutls_session ssl, response_type type, const char *str)
+{
+  response_header resp;
+
+  resp.type = type;
+  resp.data_len = strlen (str) + 1;
+  if (gnutls_record_send (ssl, &resp, sizeof (resp)) <= 0)
+    return -1;
+  if (gnutls_record_send (ssl, str, resp.data_len) <= 0)
+    return -1;
+
+  return 0;
+}
+
+
+static ssize_t
+safe_read (gnutls_session ssl, void *data, size_t count, int timeout)
+{
+  struct pollfd conn;
+  void *sock;
+
+  sock = gnutls_transport_get_ptr (ssl);
+  conn.fd = (long) sock;
+  conn.events = POLLRDNORM;
+
+  errno = 0;
+
+  while (1)
+    {
+      int nr = poll (&conn, 1, timeout * 1000);
+
+      if (nr < 0)
+        {
+	  /* Don't print error messages if poll is only interupted
+	     by a signal.  */
+	  if (errno != EINTR)
+	    dbg_log ("poll() failed: %s", strerror (errno));
+          continue;
+        }
+      else if (nr == 0)
+	{
+	  /* XXX TIMEOUT */
+	  errno = ETIME;
+	  return -1;
+	}
+      else
+	break;
+    }
+
+  /* We have new incoming data.  */
+  if (conn.revents & (POLLRDNORM|POLLERR|POLLHUP|POLLNVAL))
+    {
+      if (gnutls_record_recv (ssl, data, count) <= 0)
+	return -1;
+      else
+	return count;
+    }
+  else
+    {
+      /* I don't know if this can ever happen.  */
+      errno = EAGAIN;
+      return -1;
+    }
+}
+
+#else
+
 static int
 send_string (SSL *ssl, response_type type, const char *str)
 {
@@ -353,9 +430,14 @@ safe_read (SSL *ssl, void *data, size_t count, int timeout)
       return -1;
     }
 }
+#endif
 
 static int
+#ifdef USE_GNUTLS
+read_string (gnutls_session ssl, char **retstr)
+#else
 read_string (SSL *ssl, char **retstr)
+#endif
 {
   conv_header resp;
   *retstr = NULL;
@@ -407,7 +489,11 @@ rpasswd_conv(int num_msg, const struct pam_message **msgm,
 {
   int count=0;
   struct pam_response *reply;
+#ifdef USE_GNUTLS
+  gnutls_session ssl = appdata_ptr;
+#else
   SSL *ssl = appdata_ptr;
+#endif
 
   if (num_msg <= 0)
     return PAM_CONV_ERR;
@@ -523,7 +609,12 @@ sane_locale (const char *name)
 
 /* Handle new request.  */
 static int
+#ifdef USE_GNUTLS
+handle_request (gnutls_session ssl, request_header *req, char *locale,
+		char *username)
+#else
 handle_request (SSL *ssl, request_header *req, char *locale, char *username)
+#endif
 {
   const struct pam_conv conv = {
     rpasswd_conv,
@@ -701,8 +792,13 @@ cannot handle request version %d; current version is %d"),
 
     resp.type = FINISH;
     resp.data_len = 1;
+#ifdef USE_GNUTLS
+    gnutls_record_send (ssl, &resp, sizeof (resp));
+    gnutls_record_send (ssl, &retval, resp.data_len);
+#else
     SSL_write (ssl, &resp, sizeof (resp));
     SSL_write (ssl, &retval, resp.data_len);
+#endif
   }
 
   if (debug_level > 0)
@@ -710,10 +806,46 @@ cannot handle request version %d; current version is %d"),
   return retval;
 }
 
+#ifdef USE_GNUTLS
+/* These are global */
+static gnutls_certificate_credentials x509_cred;
+
+static gnutls_session
+initialize_gnutls_session (const char *certificate, const char *privatekey)
+{
+  gnutls_session session;
+
+  gnutls_certificate_allocate_credentials (&x509_cred);
+#if 0 /* XXX */
+  gnutls_certificate_set_x509_trust_file (x509_cred, certificate,
+					  GNUTLS_X509_FMT_PEM);
+
+  gnutls_certificate_set_x509_crl_file (x509_cred, CRLFILE,
+					GNUTLS_X509_FMT_PEM);
+#endif
+  gnutls_certificate_set_x509_key_file (x509_cred, certificate, privatekey,
+					GNUTLS_X509_FMT_PEM);
+
+  gnutls_init (&session, GNUTLS_SERVER);
+
+  gnutls_set_default_priority (session);
+  gnutls_credentials_set (session, GNUTLS_CRD_CERTIFICATE, x509_cred);
+
+  /* XXX gnutls_certificate_server_set_request (session, GNUTLS_CERT_REQUEST);
+     gnutls_dh_set_prime_bits (session, DH_BITS);
+
+     generate_dh_params();
+
+     gnutls_certificate_set_dh_params( x509_cred, dh_params); */
+
+  return session;
+}
+#endif
+
 /* This is the main loop.  It can be replicated in different threads but the
    `poll' call makes sure only one thread handles an incoming connection.  */
 static void
-server_run (void)
+server_run (const char *certificate, const char *privatekey)
 {
   int i;
 
@@ -744,8 +876,12 @@ server_run (void)
 	      int fd, ssl_err;
 	      request_header req;
 	      char buf[256];
-	      SSL *ssl;
 	      char *locale, *username;
+#ifdef USE_GNUTLS
+	      gnutls_session ssl;
+#else
+	      SSL *ssl;
+#endif
 
 	      fd = accept (pollfd_conn[i].fd, NULL, NULL);
 	      if (fd < 0)
@@ -754,8 +890,41 @@ server_run (void)
 			   strerror_r (errno, buf, sizeof (buf)));
 		  continue;
 		}
-
 	      /* TCP connection is ready. Do server side SSL. */
+#ifdef USE_GNUTLS
+	      ssl = initialize_gnutls_session (certificate, privatekey);
+	      gnutls_transport_set_ptr (ssl, (gnutls_transport_ptr)fd);
+	      ssl_err = gnutls_handshake (ssl);
+	      if (ssl_err < 0)
+		{
+		  close (fd);
+		  gnutls_deinit (ssl);
+		  dbg_log (_("Handshake has failed (%s)"),
+			   gnutls_strerror (ssl_err));
+		  continue;
+		}
+
+	      /* see the Getting peer's information example */
+	      /* print_info(session); */
+	      /* XXX print cipher info and check client
+		 certificate */
+
+	      if ((ssl_err = gnutls_record_recv (ssl, &req, sizeof (req))) <= 0)
+		{
+		  if (debug_level > 0)
+		    {
+		      if (ssl_err == 0)
+			dbg_log (_("error while reading request: %s"),
+				 _("client has closed the GNUTLS connection"));
+		      else
+			dbg_log (_("error while reading request: %s"),
+				 gnutls_strerror (ssl_err));
+		    }
+		  gnutls_deinit (ssl);
+		  close (fd);
+		  continue;
+		}
+#else
 	      ssl = SSL_new (ctx);
 	      if (ssl == NULL)
 		{
@@ -803,7 +972,8 @@ server_run (void)
 		}
 	      else
 		printf ("Client does not have certificate.\n");
-#endif	      /* Now read the request.  */
+#endif
+	      /* Now read the request.  */
 	      errno = 0;
 	      if (TEMP_FAILURE_RETRY (SSL_read (ssl, &req, sizeof (req)))
 		  != sizeof (req))
@@ -820,6 +990,7 @@ server_run (void)
 		  close (fd);
 		  continue;
 		}
+#endif
 
 	      /* It should not be possible to crash the rpasswdd with
 		 a silly request (i.e., a terribly large key). We limit
@@ -938,8 +1109,15 @@ server_run (void)
 			dbg_log (cp);
 			send_string (ssl, ERROR_MSG, cp);
 			free (cp);
+#ifdef USE_GNUTLS
+			/* do not wait for the peer to close the connection.  */
+			gnutls_bye (ssl, GNUTLS_SHUT_WR);
+			close (fd);
+			gnutls_deinit (ssl);
+#else
 			close (fd);
 			SSL_free (ssl);
+#endif
 			if (locale)
 			  free (locale);
 			free (username);
@@ -948,8 +1126,15 @@ server_run (void)
 		  break;
 		default:
 		  /* Parent: we are done.  */
+#ifdef USE_GNUTLS
+		  /* do not wait for the peer to close the connection.  */
+		  gnutls_bye (ssl, GNUTLS_SHUT_WR);
+		  close (fd);
+		  gnutls_deinit (ssl);
+#else
 		  close (fd);
 		  SSL_free (ssl);
+#endif
 		  if (locale)
 		    free (locale);
 		  free (username);
@@ -1025,7 +1210,9 @@ main (int argc, char **argv)
   int slp_timeout = 3600;
   char *slp_descr = NULL;
 #endif
+#ifndef USE_GNUTLS
   SSL_METHOD *meth;
+#endif
 
   /* Set locale via LC_ALL.  */
   setlocale (LC_ALL, "C");
@@ -1122,6 +1309,12 @@ main (int argc, char **argv)
   if (check_pid (_PATH_RPASSWDDPID))
     error (EXIT_FAILURE, 0, _("already running"));
 
+#ifdef USE_GNUTLS
+
+  /* this must be called once in the program.  */
+  gnutls_global_init ();
+
+#else
   /* Initialize SSL data. We need to do this before we go in
      background, else we cannot read the PEM phass phrase.  */
   SSL_load_error_strings ();
@@ -1152,6 +1345,7 @@ main (int argc, char **argv)
       dbg_log (ERR_error_string (ERR_get_error (), NULL));
       return E_SSL_FAILURE;
     }
+#endif
 
   /* Behave like a daemon.  */
   if (go_background)
@@ -1220,7 +1414,7 @@ main (int argc, char **argv)
 #endif
 
   /* Handle incoming requests.  */
-  server_run ();
+  server_run (certificate, privatekey);
 
   return E_SUCCESS;
 }
