@@ -1344,7 +1344,7 @@ main (int argc, char **argv)
     {
       sec_log (program, MSG_USER_NAME_INVALID,
 	       new_account, getuid ());
-      fprintf (stderr, _("%s: Invalid user name `%s'.\n"),
+      fprintf (stderr, _("%s: Invalid account name `%s'.\n"),
 	       program, new_account);
       return E_BAD_ARG;
     }
@@ -1365,7 +1365,7 @@ main (int argc, char **argv)
 	   (have_extrapath && files_getpwnam (new_account) != NULL))
     {				/* User does already exists.  */
       sec_log (program, MSG_USER_ALREADY_EXISTS, new_account, getuid ());
-      fprintf (stderr, _("%s: User `%s' already exists.\n"),
+      fprintf (stderr, _("%s: Account `%s' already exists.\n"),
 	       program, new_account);
       return E_NAME_IN_USE;
     }
@@ -1584,9 +1584,8 @@ main (int argc, char **argv)
 	      else
 		{
 		  sec_log (program, MSG_USER_ADDED_TO_GROUP,
-			   pw_data.pw.pw_name, pw_data.pw.pw_uid,
-			   gr_data->gr.gr_name, gr_data->gr.gr_gid,
-			   getuid ());
+			   pw_data.pw.pw_name, gr_data->gr.gr_name,
+			   gr_data->gr.gr_gid, getuid ());
 		}
 	    }
 	  free_group_t (gr_data);
