@@ -207,30 +207,6 @@ in_use_by_other_users (const char *dir, const char *user,
   return retval;
 }
 
-static char **
-remove_gr_mem (const char *name, char **gr_mem)
-{
-  char **groups;
-  unsigned int i, j;
-
-  for (i = 0; gr_mem[i]; i++) ;
-  ++i;                          /* for trailing NULL pointer */
-
-  groups = malloc (i * sizeof (char *));
-  for (i = 0, j = 0; gr_mem[i]; i++)
-    {
-      if (strcmp (name, gr_mem[i]) != 0)
-	{
-	  groups[j] = strdup (gr_mem[i]);
-	  ++j;
-	}
-    }
-
-  groups[j] = NULL;
-
-  return groups;
-}
-
 static int
 remove_from_secondary_groups (user_t *pw_data, int have_extrapath)
 {
