@@ -50,6 +50,7 @@
 
 #include "public.h"
 #include "group.h"
+#include "logging.h"
 #include "utf8conv.h"
 #include "logindefs.h"
 #include "read-files.h"
@@ -416,6 +417,10 @@ rename_in_secondary_groups (user_t *pw_data, int have_extrapath)
             {
               if (pw_data->binddn == NULL)
                 {
+		  sec_log (program, MSG_ERROR_RENAME_USER_IN_GROUP,
+			   pw_data->pw.pw_name, pw_data->pw.pw_uid,
+			   gr_data->gr.gr_name,
+			   gr_data->gr.gr_gid, getuid ());
                   fprintf (stderr,
                            _("%s: User not renamed in LDAP group `%s'.\n"),
                            program, utf8_to_locale (gr_data->gr.gr_name));
@@ -429,6 +434,10 @@ rename_in_secondary_groups (user_t *pw_data, int have_extrapath)
 
               if (pw_data->oldclearpwd == NULL)
                 {
+		  sec_log (program, MSG_ERROR_RENAME_USER_IN_GROUP,
+			   pw_data->pw.pw_name,
+			   pw_data->pw.pw_uid, gr_data->gr.gr_name,
+			   gr_data->gr.gr_gid, getuid ());
 		  fprintf (stderr,
 			   _("%s: User not renamed from LDAP group `%s'.\n"),
 			   program, utf8_to_locale (gr_data->gr.gr_name));
@@ -447,11 +456,22 @@ rename_in_secondary_groups (user_t *pw_data, int have_extrapath)
 					       pw_data->new_name);
           if (write_group_data (gr_data, 1) != 0)
             {
+	      sec_log (program, MSG_ERROR_RENAME_USER_IN_GROUP,
+		       pw_data->pw.pw_name,
+		       pw_data->pw.pw_uid, gr_data->gr.gr_name,
+		       gr_data->gr.gr_gid, getuid ());
               fprintf (stderr,
                        _("%s: User not renamed in group `%s'.\n"),
                        program, utf8_to_locale (gr_data->gr.gr_name));
               retval = E_GRP_UPDATE;
             }
+	  else
+	    {
+	      sec_log (program, MSG_USER_RENAMED_IN_GROUP,
+		       pw_data->new_name, pw_data->pw.pw_name,
+		       pw_data->pw.pw_uid, gr_data->gr.gr_name,
+		       gr_data->gr.gr_gid, getuid ());
+	    }
 
           item = item->next;
         }
@@ -550,31 +570,6 @@ move_home_directory (const char *oldhome, const char *newhome)
   return 0;
 }
 
-/* XXX move into libpwdutils  */
-static char **
-remove_gr_mem (const char *name, char **gr_mem)
-{
-  char **groups;
-  unsigned int i, j;
-
-  for (i = 0; gr_mem[i]; i++) ;
-  ++i;                          /* for trailing NULL pointer */
-
-  groups = malloc (i * sizeof (char *));
-  for (i = 0, j = 0; gr_mem[i]; i++)
-    {
-      if (strcmp (name, gr_mem[i]) != 0)
-        {
-          groups[j] = strdup (gr_mem[i]);
-          ++j;
-        }
-    }
-
-  groups[j] = NULL;
-
-  return groups;
-}
-
 /* XXX Make this generic and put it into libpwdutils.  */
 static int
 remove_from_secondary_groups (user_t *pw_data, int have_extrapath,
@@ -660,6 +655,10 @@ remove_from_secondary_groups (user_t *pw_data, int have_extrapath,
 
               if (pw_data->binddn == NULL)
                 {
+		  sec_log (program, MSG_ERROR_REMOVE_USER_FROM_GROUP,
+			   pw_data->pw.pw_name, pw_data->pw.pw_uid,
+			   gr_data->gr.gr_name,
+			   gr_data->gr.gr_gid, getuid ());
                   fprintf (stderr,
                            _("%s: User not removed from LDAP group `%s'.\n"),
                            program, utf8_to_locale (gr_data->gr.gr_name));
@@ -679,6 +678,10 @@ remove_from_secondary_groups (user_t *pw_data, int have_extrapath,
                     pw_data->oldclearpwd = strdup (cp);
                   else
                     {
+		      sec_log (program, MSG_ERROR_REMOVE_USER_FROM_GROUP,
+			       pw_data->pw.pw_name, pw_data->pw.pw_uid,
+			       gr_data->gr.gr_name,
+			       gr_data->gr.gr_gid, getuid ());
                       fprintf (stderr,
                                _("%s: User not removed from LDAP group `%s'.\n"),
                                program, utf8_to_locale (gr_data->gr.gr_name));
@@ -697,11 +700,21 @@ remove_from_secondary_groups (user_t *pw_data, int have_extrapath,
                                                gr_data->gr.gr_mem);
           if (write_group_data (gr_data, 1) != 0)
             {
+	      sec_log (program, MSG_ERROR_REMOVE_USER_FROM_GROUP,
+		       pw_data->pw.pw_name, pw_data->pw.pw_uid,
+		       gr_data->gr.gr_name,
+		       gr_data->gr.gr_gid, getuid ());
               fprintf (stderr,
                        _("%s: User not removed from group `%s'.\n"),
                        program, utf8_to_locale (gr_data->gr.gr_name));
               retval = E_GRP_UPDATE;
             }
+	  else
+	    {
+	      sec_log (program, MSG_USER_REMOVED_FROM_GROUP,
+		       pw_data->pw.pw_name, gr_data->gr.gr_name,
+		       gr_data->gr.gr_gid, getuid ());
+	    }
 
           item = item->next;
         }
@@ -754,6 +767,8 @@ main (int argc, char **argv)
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
+  open_sec_log ();
+
   /* Before going any further, raise the ulimit and ignore
      signals.  */
 
@@ -1010,13 +1025,18 @@ main (int argc, char **argv)
 
       if (!pw)
 	{
+	  sec_log (program, MSG_NO_ACCOUNT_FOUND, getuid ());
 	  fprintf (stderr, _("%s: Cannot determine your user name.\n"),
 		   program);
 	  return E_UNKNOWN_USER;
 	}
 
       if (do_authentication ("shadow", pw->pw_name, NULL) != 0)
-	return E_NOPERM;
+        {
+          sec_log (program, MSG_PERMISSION_DENIED, pw->pw_name,
+                   pw->pw_uid, getuid ());
+	  return E_NOPERM;
+	}
     }
 
   modify_account = locale_to_utf8 (argv[0]);
@@ -1024,7 +1044,8 @@ main (int argc, char **argv)
   pw_data = do_getpwnam (modify_account, use_service);
   if (pw_data == NULL || pw_data->service == S_NONE)
     {				/* User does not exist.  */
-      fprintf (stderr, _("%s: user `%s' does not exist.\n"),
+      sec_log (program, MSG_UNKNOWN_USER, modify_account, getuid ());
+      fprintf (stderr, _("%s: Account `%s' does not exist.\n"),
 	       program, utf8_to_locale (modify_account));
       return E_NOTFOUND;
     }
@@ -1220,6 +1241,8 @@ main (int argc, char **argv)
   if ((new_login || know_uid || new_home) &&
       is_logged_in (modify_account))
     {
+      sec_log (program, MSG_ACCOUNT_IN_USE, modify_account,
+	       pw_data->pw.pw_uid, getuid ());
       fprintf (stderr, _("%s: account `%s' is currently in use.\n"),
 	       program, utf8_to_locale (modify_account));
       return E_USER_BUSY;
@@ -1231,6 +1254,7 @@ main (int argc, char **argv)
       if (getpwuid (new_uid) != NULL ||
 	  (have_extrapath && files_getpwuid (new_uid) != NULL))
 	{
+	  sec_log (program, MSG_UID_NOT_UNIQUE, new_uid, getuid ());
 	  fprintf (stderr, _("%s: UID %u is not unique.\n"),
 		   program, new_uid);
 	  return E_UID_IN_USE;
@@ -1243,6 +1267,8 @@ main (int argc, char **argv)
     {
       if (check_name (new_login) != 0)
 	{
+	  sec_log (program, MSG_USER_NAME_INVALID,
+		   new_login, getuid ());
 	  fprintf (stderr, _("%s: Invalid account name `%s'.\n"),
 		   program, utf8_to_locale (new_login));
 	  return E_BAD_ARG;
@@ -1252,6 +1278,8 @@ main (int argc, char **argv)
 	  if (getpwnam (new_login) != NULL ||
 	      (have_extrapath && files_getpwnam (new_login) != NULL))
 	    {
+	      sec_log (program, MSG_USER_ALREADY_EXISTS,
+		       new_login, getuid ());
 	      fprintf (stderr, _("%s: Account `%s' already exists.\n"),
 		       program, utf8_to_locale (new_login));
 	      return E_NAME_IN_USE;
@@ -1264,6 +1292,7 @@ main (int argc, char **argv)
   if ((use_service == NULL || strcmp (use_service, "files") == 0) &&
       lock_database () != 0)
     {
+      sec_log (program, MSG_PASSWD_FILE_ALREADY_LOCKED);
       fputs (_("Cannot lock password file: already locked.\n"), stderr);
       return E_PWDBUSY;
     }
@@ -1354,7 +1383,51 @@ main (int argc, char **argv)
 #endif
 
       if (write_user_data (pw_data, 1) != 0)
-	return E_FAILURE;
+	{
+	  sec_log (program, MSG_ERROR_MODIFYING_USER,
+		   pw_data->pw.pw_name, pw_data->pw.pw_uid,
+		   getuid())
+	    return E_FAILURE;
+	}
+      else
+	{
+	  if (new_login)
+	    sec_log (program, MSG_USER_NAME_CHANGED,
+		     pw_data->new_name, pw_data->pw.pw_name,
+		     pw_data->pw.pw_uid, getuid());
+#if 0 /* XXX */
+	  if (new_password)
+	    LAUS_LOG(NO_TAG, "%s: user password changed - user=%s, uid=%u, by=%u",
+		     program, pw_data->pw.pw_name, pw_data->pw.pw_uid, getuid());
+	  if (pw_data->new_uid)
+	    LAUS_LOG(NO_TAG, "%s: user uid changed - user=%s, uid=%u, olduid=%u, by=%u",
+		     program, pw_data->pw.pw_name, pw_data->new_uid, pw_data->pw.pw_uid, getuid());
+	  if (new_comment)
+	    LAUS_LOG(NO_TAG, "%s: user comment changed - user=%s, uid=%u, comment='%s', oldcomment='%s', by=%u",
+		     program, pw_data->pw.pw_name, pw_data->pw.pw_uid, pw_data->new_gecos, pw_data->pw.pw_gecos,
+		     getuid());
+	  if (new_group)
+	    LAUS_LOG(NO_TAG, "%s: user default gid changed - user=%s, uid=%u, gid=%u, oldgid=%u, by=%u",
+		     program, pw_data->pw.pw_name, pw_data->pw.pw_uid, pw_data->new_gid,
+		     pw_data->pw.pw_gid , getuid());
+	  if (new_home)
+	    LAUS_LOG(NO_TAG, "%s: user home directory changed - user=%s, uid=%u, home=%s, oldhome=%s, by=%u",
+		     program, pw_data->pw.pw_name, pw_data->pw.pw_uid, pw_data->new_home, pw_data->pw.pw_dir,
+		     getuid());
+	  if (new_shell)
+	    LAUS_LOG(NO_TAG, "%s: user shell changed - user=%s, uid=%u, shell=%s, oldshell=%s, by=%u",
+		     program, pw_data->pw.pw_name, pw_data->pw.pw_uid, pw_data->new_shell, pw_data->pw.pw_shell,
+		     getuid());
+	  if (know_inactive)
+	    LAUS_LOG(NO_TAG, "%s: user inactive days changed - user=%s, uid=%u, inactive=%li, oldinactive=%li, by=%u",
+		     program, pw_data->pw.pw_name, pw_data->pw.pw_uid, pw_data->spn.sp_inact, pw_data->sp.sp_inact,
+		     getuid());
+	  if (know_expire)
+	    LAUS_LOG(NO_TAG, "%s: user expiration date changed - user=%s, uid=%u, expire=%s, oldexpire=%s, by=%u",
+		     program, pw_data->pw.pw_name, pw_data->pw.pw_uid, date2str(DAY*pw_data->spn.sp_expire),
+		     date2str(DAY*pw_data->sp.sp_expire), getuid());
+#endif
+        }
 
 #ifdef HAVE_NSCD_FLUSH_CACHE
       /* flush NSCD cache, else later calls could get obsolete data.  */
@@ -1473,6 +1546,12 @@ main (int argc, char **argv)
 			   utf8_to_locale (gr_data->gr.gr_name));
 		  retval = E_GRP_UPDATE;
 		}
+	      else
+		{
+		  sec_log (program, MSG_USER_ADDED_TO_GROUP,
+                           pw_data->pw.pw_name, gr_data->gr.gr_name,
+                           gr_data->gr.gr_gid, getuid ());
+		}
 	      /* Make sure, written group changes will be active now.  */
 	      nscd_flush_cache ("group");
 	    }
@@ -1567,6 +1646,12 @@ main (int argc, char **argv)
 			   utf8_to_locale (gr_data->gr.gr_name));
 		  retval = E_GRP_UPDATE;
 		}
+	      else
+		{
+		  sec_log (program, MSG_USER_ADDED_TO_GROUP,
+                           pw_data->pw.pw_name, gr_data->gr.gr_name,
+                           gr_data->gr.gr_gid, getuid ());
+		}
 	      /* Make sure, written group changes will be active now.  */
 	      nscd_flush_cache ("group");
 	    }
